@@ -1,10 +1,8 @@
-# Copyright (c) 2016 The Regents of the University of Michigan
+# Copyright (c) 2017 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
 from . import six
-
-class Error(Exception):
-    pass
+from ..core.errors import Error
 
 
 class ConfigError(Error, RuntimeError):
