@@ -19,6 +19,9 @@ from .indexing import export_one
 from .indexing import export
 from .indexing import export_to_mirror
 from .indexing import export_pymongo
+from .indexing import index_files
+from .indexing import index
+from .collection import Collection
 
 logger = logging.getLogger(__name__)
 
@@ -29,6 +32,8 @@ __all__ = [
     'BaseCrawler', 'RegexFileCrawler', 'JSONCrawler', 'SignacProjectCrawler',
     'MasterCrawler', 'fetch', 'fetch_one', 'fetched',
     'export_one', 'export', 'export_to_mirror', 'export_pymongo',
+    'index_files', 'index',
+    'Collection',
 ]
 
 
